#include <iostream>
#include <cstdio>
#include <cmath>
#include <vector>
#include <algorithm>
#include <set>
#include <queue>

using namespace std;

#define X			first
#define Y			second
#define endl		'\n'

struct rect {
	int x1, y1;
	int x2, y2;
	rect() {}
	rect( int _x1, int _y1, int _x2, int _y2 ) {
		x1 = _x1;
		y1 = _y1;
		x2 = _x2;
		y2 = _y2;
	}
};

vector< int > prime;

void getPrime( int h, int w ) {
	prime.clear();
	vector< bool > u( 32000, true );
	for( int i = 2; i < 32000; i ++ ) {
		if( !u[i] )
			continue;
		for( int j = i * i; j < 32000; j += i )
			u[j] = false;
		if( h % i == 0 || w % i == 0 )
			continue;
		prime.push_back( i );
	}
}

bool in( int x, int n ) {
	return ( 0 <= x && x < n );
}

bool in( int x, int y, int n, int m ) {
	return in( x, n ) && in( y, m );
}

void writeTest( int h, int w, int n, int m, vector< string > &s, int q, vector< rect > &Q ) {
	cin.tie( 0 ); cin.sync_with_stdio( 0 );
	freopen( "input.txt", "wt", stdout );

	cout << h << " " << w << endl;
	cout << n << " " << m << endl;
	for( int i = 0; i < n; i ++ ) {
		cout << s[i] << endl;
	}
	cout << q << endl;
	for( int i = 0; i < q; i ++ ) {
		cout << Q[i].x1 << " " << Q[i].y1 << " " << Q[i].x2 << " " << Q[i].y2 << endl;
	}
	cout << endl;
}

int randNumber( int l, int r ) {
	int x = 1;
	x *= prime[rand() % prime.size()];
	x *= prime[rand() % prime.size()];
	x %= ( r - l + 1 );
	x += l;
	return x;
}

void genMaxNumber( int &x, int start, int mx ) {
	x = start;
	while( x * 2 <= mx ) {
		x *= 2;
	}
}

vector< string > genMap( int n, int m, int p ) {
	vector< string > s( n, "" );
	for( int i = 0; i < n; i ++ ) {
		for( int j = 0; j < m; j ++ ) {
			if( randNumber( 1, 100 ) <= p )
				s[i] += 'X';
			else
				s[i] += '.';
		}
	}
	return s;
}

vector< rect > genQAll( int h, int w, int q ) {
	vector< rect > Q;
	for( int i = 1; i <= h; i ++ ) {
		for( int j = 1; j <= w; j ++ ) {
			Q.push_back( rect( i, j, i, j ) );
		}
	}
	random_shuffle( Q.begin(), Q.end() );
	while( Q.size() > q ) {
		Q.pop_back();
	}
	return Q;
}

bool check( int x1, int y1, int x2, int y2, int d ) {
	long long dx = x2 - x1 + 1;
	long long dy = y2 - y1 + 1;
	return d >= dx * dy;
}

rect randRect( int h, int w, int d, int dx, int dy ) {
	if( d == 0 ) {
		int x = randNumber( 1, h );
		int y = randNumber( 1, w );
		return rect( x, y, x, y );
	}
	if( d == -1 ) {
		int x1 = randNumber( 1, h - dx );
		int y1 = randNumber( 1, w - dy );
		int x2 = randNumber( x1 + dx, h );
		int y2 = randNumber( y1 + dy, w );
		return rect( x1, y1, x2, y2 );
	}
	int DX = d / ( dy + 1 ) - 1;
	int x1 = randNumber( 1, h - DX );
	int x2 = randNumber( x1 + dx, x1 + DX );
	int DY = d / ( x2 - x1 + 1 ) - 1;
	int y1 = randNumber( 1, w - DY );
	int y2 = randNumber( y1 + dy, y1 + DY );
	if( !check( x1, y1, x2, y2, d ) )
		exit( 1 );
	return rect( x1, y1, x2, y2 );
}

vector< rect > genQ( int h, int w, int q, int d, int dx, int dy ) {
	if( d == 0 && 1LL * h * w <= 1000000 )
		return genQAll( h, w, q );
	vector< rect > Q;
	for( int i = 0; i < q; i ++ ) {
		Q.push_back( randRect( h, w, d, dx, dy ) );
	}
	return Q;
}

int main( int argc, char *argv[] ) {
	int test = atol( argv[1] );
	int n = atol( argv[2] );
	int m = atol( argv[3] );
	int maxH = atol( argv[4] );
	int maxW = atol( argv[5] );
	int p = atol( argv[6] );
	int q = atol( argv[7] );
	int d = atol( argv[8] );
	int dx = atol( argv[9] );
	int dy = atol( argv[10] );

	getPrime( maxH, maxW );

	if( n < 0 )
		n = randNumber( abs( n ), min( abs( n ) * 3, 1000 ) );
	if( m < 0 )
		m = randNumber( abs( m ), min( abs( m ) * 3, 1000 ) );

	if( rand() % 2 )
		swap( n, m );

	srand( test + n + m + p + q + d + dx + dy );

	int h, w;
	genMaxNumber( h, n, maxH );
	genMaxNumber( w, m, maxW );

	getPrime( h, w );

	if( 1LL * h * w <= q )
		q = h * w;

	if( dx >= h )
		dx = h - 100;
	if( dy >= w )
		dy = w - 100;

	if( d > 0 && 1LL * ( dx + 1 ) * ( dy + 1 ) > d )
		dy = d / dx - 100;

	dx = max( 0, dx );
	dy = max( 0, dy );

	vector< string > s = genMap( n, m, p );
	vector< rect > Q = genQ( h, w, q, d, dx, dy );

	writeTest( h, w, n, m, s, q, Q );
    return 0;
}
